/*************************************************************************
  Copyright (c) 1993 - 1997
  Digital Equipment Corporation, Maynard, Mass.

  This software is furnished under a license and may be used and  copied
  only  in  accordance  with  the  terms  of  such  license and with the
  inclusion of the above copyright notice.  This software or  any  other
  copies  thereof may not be provided or otherwise made available to any
  other person.  No title to and ownership of  the  software  is  hereby
  transferred.

  The information in this software is subject to change  without  notice
  and  should  not  be  construed  as  a commitment by DIGITAL Equipment
  Corporation.
 *************************************************************************/


/*++
* ============
* = vcdump.c =
* ============
*
* Description:
*   This application dumps the vc-specific information that each
*   installed ATM driver registers with the CMM.
*
* Arguments:
*   none.
*
* Authors:
*   TLR - Theodore L. Ross
*
* Modification History:
*   Date          Name     Description
*   22-Sep-1996   TLR      Created.
*
* Issues:
*   none.
--*/

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <sys/time.h>
#include <sys/select.h>
#include <netaal/aal.h>

#define SIZE 18190

main (int argc, char **argv)
   {
   int                  sock;
   int                  a, i, j, cnt = 0;
   unsigned long long   bytes = 0;
   unsigned char        buff[32768];
   unsigned int         device_count;
   ATM_CONNECTION_ID    conn_id;
   struct timespec      start_time, curr_time;

   /* Prevent garbage array data by clearing out the buff array. */

   for (i = 0; i < 32768; i++)
      buff[i] = '\0';

   /* The user must pass exactly three arguments.  The executable, device number,
   *  and the vci desired. 
   */

   if (argc != 3)
      {
      printf ("Usage: %s device# vci\n", argv[0]);
      printf ("   where device# is the instance value and \n");
      printf ("         vci# is the vci to interrogate\n");
      exit (1);
      }

   /* Scan the user's command-line parameters and convert into the correct
   *  type.  Note that by default the vpi is 0.
   */

   sscanf (argv[1], "%d", &conn_id.device_number);
   sscanf (argv[2], "%d", &conn_id.vci);
   conn_id.vpi = 0;

   /* Open an AAL socket for DGRAM service. */

   printf ("[ Creating ATM Socket ]\n");
   sock = socket (AF_AAL, SOCK_DGRAM, 0);
   if (sock == -1)
      {
      printf ("[ socket call failed, error: %d ]\n", errno);
      exit (1);
      }

   /* Get the current number of ATM devices. */

   if (ioctl (sock, SIO_GET_ATM_DEVICE_COUNT, &device_count) == -1)
      {
      printf ("[ SIO_GET_ATM_DEVICE_COUNT ioctl failed, error: %d ]\n",
              errno);
      exit (1);
      }

   /* If the user's instance number exceeds the device count, return an
   *  error.  It is not possible to connect to this device. 
   */

   if (conn_id.device_number >= device_count)
      {
      printf ("Invalid Device Number.\n");
      exit (1);
      }

   /* Associate a PVC to this vci and device. */

   if (ioctl (sock, SIO_ASSOCIATE_PVC, &conn_id) == -1)
      {
      printf ("[ SIO_ASSOCIATE_PVC ioctl failed, error: %d ]\n", errno);
      exit (1);
      }


   printf ("[ Listening on Socket... ]\n");
   getclock (TIMEOFDAY, &start_time);

   while (1)
      {
      fd_set              readfds;
      struct timeval      wait;
      unsigned long long  timestamp;
      unsigned int        minutes, seconds, milliseconds;

      FD_ZERO (&readfds);

      wait.tv_sec  = 1;
      wait.tv_usec = 0;

      FD_SET  (sock, &readfds);
      a = select (4, &readfds, NULL, NULL, &wait);
      while (a == 0)
         {
         FD_SET  (sock, &readfds);
         a = select (4, &readfds, NULL, NULL, &wait);
         }        

      if (a == -1)
         {
         printf ("[ select failed, error: %d ]\n", errno);
         exit (1);
         }

      i = recv (sock, buff, SIZE, 0);
      cnt++;

      getclock (TIMEOFDAY, &curr_time);
      timestamp =  /* time in milliseconds */
         ((curr_time.tv_sec  * 1000) + (curr_time.tv_nsec  / 1000000)) -
         ((start_time.tv_sec * 1000) + (start_time.tv_nsec / 1000000));

      minutes      = timestamp / 60000;
      seconds      = (timestamp - (minutes * 60000)) / 1000;
      milliseconds = (timestamp - (minutes * 60000) - (seconds * 1000));

      printf ("Time: %02d:%02d.%03d, Count: %d, VCI: %d, length: %d\n",
              minutes, seconds, milliseconds,
              cnt, conn_id.vci, i);
      
      for (j = 0; j < MIN (i, 512); j++)
         {
         printf ("%02x ", buff[j]);
         if ((j % 16) == 15)
            printf ("\n");
         }
      if ((j % 16) != 15)
         printf ("\n");
      }

   close (sock);
   }

